//GLOBALS
var DATASTORAGE = [];
var MODSTORAGE = [];

var CONTROL_TOP = document.getElementById("topControl");
var CONTROL_MID = document.getElementById("midControl");
var CONTROL_BOT = document.getElementById("botControl");

var OUTPUT_TOP = document.getElementById("topOutput");
var OUTPUT_MID = document.getElementById("midOutput");
var OUTPUT_BOT = document.getElementById("botOutput");

var NUMBER_OF_STEPS = 10000;

var ANALOG_NUM_CYCLES = 10;
var ANALOG_TIME_STEP = .001;

var CURRENT_ELEMENTS = [];

var TWOPI = 2 * Math.PI;

function loadViewer()
{
	var testData = 
	[
		{x: 0, y: 0},
		{x: 10, y: 2},
		{x: 20, y: 6},
		{x: 30, y: 10},
		{x: 40, y: 6},
		{x: 50, y: 2},
		{x: 60, y: 0},
		{x: 70, y: -7},
		{x: 80, y: -10},
		{x: 90, y: -7},
		{x: 100, y: 0}
	];


	modeChange(document.getElementById("modulation").value);
	
}

function modeChange(e)
{
	CONTROL_TOP.innerHTML = "";
	CONTROL_MID.innerHTML = "";
	CONTROL_BOT.innerHTML = "";
	
	OUTPUT_TOP.innerHTML = "";
	OUTPUT_MID.innerHTML = "";
	OUTPUT_BOT.innerHTML = "";
	
	CURRENT_ELEMENTS = [];
	
	zoomChange(document.getElementById("zoom").value);
	
	if (e == "AA")
	{
		CURRENT_ELEMENTS.push(new AnalogSignal(CONTROL_TOP, OUTPUT_TOP, "analogMessage", "Analog Message", "data", "x"));
		CURRENT_ELEMENTS.push(new AnalogSignal(CONTROL_MID, OUTPUT_MID, "analogCarrier", "Analog Carrier", "mod", "y"));
		CURRENT_ELEMENTS.push(new AAModulator(CONTROL_BOT, OUTPUT_BOT, "aaModulator", "Analog/Analog Modulation"));
	}
	else if (e == "AD")
	{	
		CURRENT_ELEMENTS.push(new AnalogSignal(CONTROL_TOP, OUTPUT_TOP, "analogMessage", "Analog Message", "data", "x"));
		CURRENT_ELEMENTS.push(new ADModulator(CONTROL_MID, OUTPUT_MID, "adModulator", "Analog/Digital Modulation"));		
	}
	else if (e == "DA")
	{	
		CURRENT_ELEMENTS.push(new DigitalSignal(CONTROL_TOP, OUTPUT_TOP, "digitalMessage", "Digital Message", "data"));
		CURRENT_ELEMENTS.push(new AnalogSignal(CONTROL_MID, OUTPUT_MID, "analogCarrier", "Analog Carrier", "mod", "y"));
		CURRENT_ELEMENTS.push(new DAModulator(CONTROL_BOT, OUTPUT_BOT, "daModulator", "Digital/Analog Modulator"));
	}
	else if (e == "DD")
	{
		CURRENT_ELEMENTS.push(new DigitalSignal(CONTROL_TOP, OUTPUT_TOP, "digitalMessage", "Digital Message", "data"));
		CURRENT_ELEMENTS.push(new DDModulator(CONTROL_MID, OUTPUT_MID, "daModulator", "Digital/Digital Modulator"));		
	}
	
	MathJax.Hub.Typeset();
	
}

function zoomChange(e)
{
	if(e == "10")
	{
		ANALOG_NUM_CYCLES = 10;
		ANALOG_TIME_STEP = .001;
	}
	else if(e == "1")
	{
		ANALOG_NUM_CYCLES = 1;
		ANALOG_TIME_STEP = .0001;	
	}
	else if(e == ".1")
	{
		ANALOG_NUM_CYCLES = .1;
		ANALOG_TIME_STEP = .00001;	
	}
	else if(e == ".01")
	{
		ANALOG_NUM_CYCLES = .01;
		ANALOG_TIME_STEP = .000001;	
	}
	
	updateElements();
}

function updateElements()
{
	for(var x = 0; x < CURRENT_ELEMENTS.length; x++)
	{
		CURRENT_ELEMENTS[x].refresh();
	}
	
	MathJax.Hub.Typeset();
}

function getTextInput(label, parentName, name, initialValue, eventHandler)
{
	var row = document.createElement("div");
	row.setAttribute("class", "formRow");
	
	var labelContainer = document.createElement("div");
	labelContainer.setAttribute("class", "formLabel");
	labelContainer.innerHTML = label;
	
	var elementContainer = document.createElement("div");
	elementContainer.setAttribute("class", "formElement");
	
	var textBox = document.createElement("input");
	textBox.setAttribute("type", "text");
	textBox.setAttribute("value", initialValue);
	textBox.setAttribute("id", parentName + "_" + name);
	textBox.setAttribute("class", "textBox");
	textBox.setAttribute("size", "5");
	textBox.addEventListener("change", eventHandler, false);
	
	elementContainer.appendChild(textBox);
	row.appendChild(labelContainer);
	row.appendChild(elementContainer);
	
	return row;
}
	
function getListInput(label, parentName, name, optionData, eventHandler)
{
	var row = document.createElement("div");
	row.setAttribute("class", "formRow");
	
	var labelContainer = document.createElement("div");
	labelContainer.setAttribute("class", "formLabel");
	labelContainer.innerHTML = label;
	
	var elementContainer = document.createElement("div");
	elementContainer.setAttribute("class", "formElement");
	
	var selectList = document.createElement("select");
	selectList.setAttribute("class", "selectList");
	selectList.setAttribute("id", parentName + "_" + name);
	selectList.addEventListener("change", eventHandler, false);
		
	for (var x = 0; x < optionData.length; x++)
	{
		var listOption = document.createElement("option");
		listOption.setAttribute("class", "listOption");
		listOption.setAttribute("value", optionData[x].value);
		listOption.innerHTML = optionData[x].label;
		
		selectList.appendChild(listOption);
	}
	
	elementContainer.appendChild(selectList);
	
	row.appendChild(labelContainer);
	row.appendChild(elementContainer);
	
	return row;
}

function getTitledBox(title)
{
	var titleDiv = document.createElement("div");
	titleDiv.setAttribute("class", "title");
	titleDiv.innerHTML = title;
	
	var containerDiv = document.createElement("div");
	containerDiv.setAttribute("class", "roundedBox");
	containerDiv.appendChild(titleDiv);
	
	return containerDiv;
}

function getSquareBox(title)
{
	var titleDiv = document.createElement("div");
	titleDiv.setAttribute("class", "title");
	titleDiv.innerHTML = title;
	
	var containerDiv = document.createElement("div");
	containerDiv.setAttribute("class", "squareBox");
	containerDiv.appendChild(titleDiv);
	
	return containerDiv;
}

function getFormRow(label)
{
	var row = document.createElement("div");
	row.setAttribute("class", "formRow");
	
	var labelContainer = document.createElement("div");
	labelContainer.setAttribute("class", "formLabel");
	labelContainer.innerHTML = label;
	
	var elementContainer = document.createElement("div");
	elementContainer.setAttribute("class", "formElement");
	
	row.appendChild(labelContainer);
	row.appendChild(elementContainer);
	
	return row;
}

function getContainer(parentName, className)
{
	var containerDiv = document.createElement("div");
	containerDiv.setAttribute("class", className);
	containerDiv.setAttribute("id", parentName + "_" + className);
	
	return containerDiv;
}

function makeEquation(eq)
{
	var container = document.createElement("span");
	container.setAttribute("class", "equation");
	
	container.innerHTML = "`" + eq + "`";
	
	return container;
}

function roundToPlace(number, place)
{
	var ret = Math.round(number * (Math.pow(10, place))) / (Math.pow(10, place));
	return ret;	
}



function SignalGraph(parentNode, data, title, analogData, xLabel, yLabel)
{	
	var graphBox = getSquareBox(title);
	var graphArea = parentNode.appendChild(graphBox);
	
	var canvasWidth = graphArea.offsetWidth;
	var canvasHeight = graphArea.offsetHeight;
	
	var margin = {top: 10, right: 45, bottom: 33, left: 50};
    var padding = {top: 0, right: 0, bottom: 32, left: 0};
    var outerWidth = canvasWidth;
    var outerHeight = canvasHeight;
    var innerWidth = outerWidth - margin.left - margin.right;
    var innerHeight = outerHeight - margin.top - margin.bottom;
    var width = innerWidth - padding.left - padding.right;
    var height = innerHeight - padding.top - padding.bottom;

	//Create X and Y axis
	
	//var axisPadding = 35;
	//var axisThickness = 50;
	
	var xAxisSize = width;
	var yAxisSize = height;
	
	var xScale = d3.scale.linear()
                .range([0, width])
                .domain([d3.min(data, function(d){return d.x}), d3.max(data, function(d){return d.x})])
				.nice();
	
	var yScale = d3.scale.linear()
                .range([height, 0])
                .domain([d3.min(data, function(d){return d.y}), d3.max(data, function(d){return d.y})])
				.nice();
				
	var xAxis = d3.svg.axis()
		.scale(xScale)
		.innerTickSize(height)
		.outerTickSize(1)
		.tickPadding(5);		
		
	var yAxis = d3.svg.axis()
		.scale(yScale)
		.orient("left")
		.innerTickSize(-width)
		.outerTickSize(1)
		.tickPadding(5);
	
	if(analogData)
	{
		yAxis.tickValues([d3.min(data, function(d){return d.y}), 0, d3.max(data, function(d){return d.y})]);
	}
	
	//Select canvas element
	var canvas = d3.select(graphArea).append("svg")
			.attr("width", width + margin.left + margin.right)
			.attr("height", height + margin.top + margin.bottom)
		.append("g")
			.attr("transform", "translate(" + margin.left + "," + margin.top + ")");
			
	//Add X and Y axis
	canvas.append("svg:g")
		.call(xAxis);
		//.attr("transform", "translate(0, " + yAxisSize + ")");
	
	canvas.append("svg:g")
		.call(yAxis);
		//.attr("transform", "translate(" + axisThickness + ", 0)");
		
	canvas.append("text")
		.attr("class", "axisLabel")
		.attr("text-anchor", "end")
		.attr("x", (.5 * width))
		.attr("y", height + margin.bottom)
		.text(xLabel);
		
	canvas.append("text")
		.attr("class", "axis label")
		.attr("text-anchor", "end")
		.attr("y", -(margin.left - 16))
		.attr("x", -.3 * outerHeight)
		.attr("transform", "rotate(-90)")
		.text(yLabel);

	//Create line
	var graphLine = d3.svg.line()
		.x(function(d){return xScale(d.x);})
		.y(function(d){return yScale(d.y);})
		.interpolate('basis');
		
	//Add line
	canvas.append('svg:path')
		.attr('d', graphLine(data))
		.attr('stroke', 'blue')
		.attr('stroke-width', 2)
		.attr('fill', 'none');
};



function AAModulator(controlParent, graphParent, name, title)
{
	this.name = name;
	this.graphParent = graphParent;
	
	var listData = [{value: "am", label: "AM"}, {value: "fm", label: "FM"}];
	
	var modInput = getListInput("Modulation", name, "modType", listData, this.inputsChanged);
	var eqCont = getContainer(name, "waveData");
	
	var panelBox = getTitledBox(title);
	panelBox.appendChild(modInput);
	panelBox.appendChild(eqCont)
		
	controlParent.appendChild(panelBox);
	
	this.dataCont = document.getElementById(name + "_" + "waveData");
	
	this.refresh();
};

AAModulator.prototype.inputsChanged = function()
{
	updateElements();
};

AAModulator.prototype.refresh = function()
{
	this.graphParent.innerHTML = "";
	
	var sigType = "";
	var sigEq = "";
	var modVar = "";
	var modEq = "";
	
	var data = [];
	var modIndex = 0;
	
	if(this.getValue("modType") == "am")
	{	
		var indexEq = DATASTORAGE.amp + " / " + MODSTORAGE.amp;
		modVar = "n_a";
	
		for(var x = 0; x <= ANALOG_NUM_CYCLES; x = x + ANALOG_TIME_STEP)
		{
			modIndex = (DATASTORAGE.amp / MODSTORAGE.amp);
			modEq = "x_a / y_a";
			sigType = "AM";
			
			var msgSig;
			if(DATASTORAGE.waveType == "sine")
			{
				msgSig = Math.sin((DATASTORAGE.freq * TWOPI * x));
			}
			else
			{
				msgSig = Math.cos((DATASTORAGE.freq * TWOPI * x));
			}
		
			var carSig;
			if(MODSTORAGE.waveType == "sine")
			{
				carSig = Math.sin((MODSTORAGE.freq * TWOPI * x));
			}
			else
			{
				carSig = Math.cos((MODSTORAGE.freq * TWOPI * x));
			}
			
			var calculation = (1 + (modIndex * msgSig)) * carSig;
		
			data.push(
			{
				x: x,
				y: calculation
			});
		}
		sigEq = "(1 + [ " + modVar + " x(2&pi;tx_f)] )y(2&pi;ty_f)";
	}
	else if(this.getValue("modType") == "fm")
	{
		modVar = "n_f";
		
		for(var x = 0; x <= ANALOG_NUM_CYCLES; x = x + ANALOG_TIME_STEP)
		{
			var modIndex = DATASTORAGE.amp / (TWOPI * DATASTORAGE.freq);
			var modEq = "x_a / (2&pi;x_f)";
			
			sigType = "FM";
			
			var msgSig;
			if(DATASTORAGE.waveType == "sine")
			{
				msgSig = Math.sin(DATASTORAGE.freq * x * TWOPI);
			}
			else
			{
				msgSig = Math.cos(DATASTORAGE.freq * x * TWOPI);
			}
			
			var carSig;
			if(MODSTORAGE.waveType == "sine")
			{
				carSig = Math.sin((MODSTORAGE.freq * x * TWOPI) + (modIndex * msgSig));
			}
			else
			{
				carSig = Math.cos((MODSTORAGE.freq * x * TWOPI) + (modIndex * msgSig));
			}

			var calculation = MODSTORAGE.amp * carSig;
		
			data.push(
			{
				x: x,
				y: calculation
			});
		}
		sigEq = "y_a(y(2&pi;ty_f + [" + modVar + "x(2&pi;tx_f)])"
	}
	
	this.dataCont.innerHTML = "";
	
	var indexRow = getFormRow("Mod Index: `" + modVar + "`");
	indexRow.childNodes[1].appendChild(makeEquation(modEq + " = " + roundToPlace(modIndex, 2)));
	var eqRow = getFormRow("Equation");
	eqRow.childNodes[1].appendChild(makeEquation(sigEq));
	
	this.dataCont.appendChild(indexRow);
	this.dataCont.appendChild(eqRow);
	
	new SignalGraph(this.graphParent, data, sigType + " Waveform", true, "Seconds", "Amplitude");	
};

AAModulator.prototype.getValue = function(name)
{
	return document.getElementById(this.name + "_" + name).value;
};



function ADModulator(controlParent, graphParent1, name, title)
{
	this.name = name;
	this.graphParent1 = graphParent1;
	
	var listData = [{value: "pcm", label: "PCM"}, {value: "dm", label: "DM"}];
	
	var sampleRate = getTextInput("Sample Rate", this.name, "sampleRate", 10, this.inputsChanged);
	var encodeSize = getTextInput("Encoding Size", this.name, "encodeSize", 4, this.inputsChanged);
	var modInput = getListInput("Modulation", name, "modType", listData, this.inputsChanged);
	var cont = getContainer(name, "bpsData");
	
	var panelBox = getTitledBox(title);
	panelBox.appendChild(modInput);
	panelBox.appendChild(sampleRate);
	panelBox.appendChild(encodeSize);
	panelBox.appendChild(cont);
	
	controlParent.appendChild(panelBox);
	
	this.modOptons = document.getElementById(name + "_modOptions");
	this.bpsCont = document.getElementById(name + "_bpsData");
	
	this.refresh();
};

ADModulator.prototype.inputsChanged = function()
{
	updateElements();
};

ADModulator.prototype.getValue = function(name)
{
	return document.getElementById(this.name + "_" + name).value;
};

ADModulator.prototype.refresh = function()
{
	this.graphParent1.innerHTML = "";
	
	var data = [];
	
	var totalCodes = Math.pow(2, this.getValue("encodeSize"));
	var bandSize = ((2 * DATASTORAGE.amp) / totalCodes);
	
	var sampleStep = (1 / this.getValue("sampleRate"));
	var sampleTarget = 0;
	
	var oldPAM = 0;	
	
	for(var x = 0; x <= ANALOG_NUM_CYCLES; x = x + ANALOG_TIME_STEP)
	{
		var pamcode;
		
		if(x >= sampleTarget)
		{
			var dataWave = 0;
			if(DATASTORAGE.waveType == "sine")
			{
				dataWave = Number(DATASTORAGE.amp) + (Number(DATASTORAGE.amp) * Math.sin(x * TWOPI * DATASTORAGE.freq));
			}
			else
			{
				dataWave = Number(DATASTORAGE.amp) + (Number(DATASTORAGE.amp) * Math.cos(x * TWOPI * DATASTORAGE.freq));
			}
		
			if(this.getValue("modType") == "pcm")
			{
				pamCode = Math.floor(dataWave / bandSize);
			}
			else if(this.getValue("modType") == "dm")
			{
				if(dataWave > (oldPAM * bandSize))
				{
					pamCode++;
				}
				else
				{
					pamCode--;
				}
			}
			
			if(pamCode >= (totalCodes - 1))
			{
				pamCode = (totalCodes - 1);
			}
			else if(pamCode < 0)
			{
				pamCode = 0;
			}

			
			
			oldPAM = pamCode;
			sampleTarget = sampleTarget + sampleStep;
		}
		else
		{
			pamCode = oldPAM;
		}
			
		
		data.push(
		{
			x: x,
			y: pamCode
		});
	}
	this.bpsCont.innerHTML = "";
	var bpsRow = getFormRow("Data Rate (BPS)");
	bpsRow.childNodes[1].innerHTML = Number(this.getValue("encodeSize")) * Number(this.getValue("sampleRate"));
	this.bpsCont.appendChild(bpsRow);
	
	new SignalGraph(this.graphParent1, data, "Reconstructed Wave", false, "Seconds", "Code");
};



function DAModulator(controlParent, graphParent, name, title)
{
	this.name = name;
	this.graphParent = graphParent;
	
	var listData = [{value: "ask", label: "ASK"}, {value: "fsk", label: "FSK"}, {value: "psk", label: "PSK"}];
	var modInput = getListInput("Modulation", name, "modType", listData, this.inputsChanged);
	
	var container = getContainer(this.name, "fskOption");
	var fDelta = getTextInput("F Delta", name, "fDelta", 5, this.inputsChanged);
	container.appendChild(fDelta);
	this.fskContainer = container;
	
	var panelBox = getTitledBox(title);
	panelBox.appendChild(modInput);
	panelBox.appendChild(container);
	
	controlParent.appendChild(panelBox);
	
	this.refresh();	
}

DAModulator.prototype.inputsChanged = function()
{
	updateElements();
};
	
DAModulator.prototype.getValue = function(name)
{
	return document.getElementById(this.name + "_" + name).value;
};
	
DAModulator.prototype.refresh = function()
{
	this.graphParent.innerHTML = "";
	
	var sigType = "";
	
	var data = [];
	var signalStep = (1 / DATASTORAGE.bps);
	var currentStep = 0;
	var currentBit = 0;
	
	
	if(this.getValue("modType") == "fsk")
	{
		this.fskContainer.style.display = "initial";
	}
	else
	{
		this.fskContainer.style.display = "none";
	}
	
	
	for(var x = 0; x <= ANALOG_NUM_CYCLES; x = x + ANALOG_TIME_STEP)
	{
		var signal = 0;	
		
		if(x >= (currentStep * signalStep))
			{
				if(Number(DATASTORAGE.data[currentStep]) == 1)
				{
					currentBit = 1;
				}
				else
				{
					currentBit = 0;
				}
				
				currentStep++;
			}		
			
		if(this.getValue("modType") == "ask")
		{
			sigType = "ASK";
			
			if(currentBit == 1)
			{
				if(MODSTORAGE.waveType == "sine")
				{
					signal = (MODSTORAGE.amp * Math.sin(TWOPI * MODSTORAGE.freq * x));
				}
				else
				{
					signal = (MODSTORAGE.amp * Math.cos(TWOPI * MODSTORAGE.freq * x));	
				}
				
			}
			else
			{
				signal = 0;
			}
		}		
		else if(this.getValue("modType") == "fsk")
		{
			sigType = "FSK";
			
			if(currentBit == 1)
			{
				var freq = Number(MODSTORAGE.freq) + Number(this.getValue("fDelta"));	
				if(MODSTORAGE.waveType == "sine")
				{
					signal = (MODSTORAGE.amp * Math.sin(TWOPI * freq * x));
				}
				else
				{					
					signal = (MODSTORAGE.amp * Math.cos(TWOPI * freq * x));	
				}				
			}
			else
			{
				var freq = MODSTORAGE.freq - this.getValue("fDelta");
				if(freq < 0)
				{
					freq = 0;
				}
				
				if(MODSTORAGE.waveType == "sine")
				{
					signal = (MODSTORAGE.amp * Math.sin(TWOPI * freq * x));
				}
				else
				{					
					signal = (MODSTORAGE.amp * Math.cos(TWOPI * freq * x));	
				}				
			}			
		}
		else if(this.getValue("modType") == "psk")
		{
			sigType = "PSK"
			
			if(currentBit == 1)
			{
				
				if(MODSTORAGE.waveType == "sine")
					{
						signal = (MODSTORAGE.amp * Math.sin(TWOPI * MODSTORAGE.freq * x));
					}
				else
					{					
						signal = (MODSTORAGE.amp * Math.cos(TWOPI * MODSTORAGE.freq * x));	
					}		
			}
			else
			{
				if(MODSTORAGE.waveType == "sine")
				{
					signal = (-1 * MODSTORAGE.amp * Math.sin(TWOPI * MODSTORAGE.freq * x));
				}
				else
				{					
					signal = (-1 * MODSTORAGE.amp * Math.cos(TWOPI * MODSTORAGE.freq * x));	
				}		
			}
		}
		
		data.push(
		{
			x: x,
			y: signal
		});	
	}
	
	new SignalGraph(this.graphParent, data, sigType + " Waveform", true, "Seconds", "Amplitude");	
};



function DDModulator(controlParent, graphParent, name, title)
{
	this.name = name;
	this.graphParent = graphParent;
		
	var listData = [{value: "nrzl", label: "NRZ-L"}, {value: "nrzi", label: "NRZI"}, {value: "bami", label: "Bipolar-AMI"}
	, {value: "pst", label: "Pseudoternary"}, {value: "manchester", label: "Manchester"}, {value: "dManchester", label: "Diff Manchester"}];
	var modInput = getListInput("Modulation", name, "modType", listData, this.inputsChanged);
	
	var panelBox = getTitledBox(title);
	panelBox.appendChild(modInput)
	
	controlParent.appendChild(panelBox);
	
	this.refresh();
}

DDModulator.prototype.inputsChanged = function()
{
	updateElements();
};
	
DDModulator.prototype.getValue = function(name)
{
	return document.getElementById(this.name + "_" + name).value;
};
	
DDModulator.prototype.refresh = function()
{
	
	
	this.graphParent.innerHTML = "";
	
	var data = [];
	
	var signalStep = (1 / Number(DATASTORAGE.bps));
	var signalTarget = 0;
	var currentStep = 0;
	var currentBit = 0;
	var lastBit = 0;
	var signal = 0;
	
	for(var x = 0; x <= ANALOG_NUM_CYCLES; x = x + ANALOG_TIME_STEP)
	{				
		if(x >= signalTarget)
		{		
			if(Number(DATASTORAGE.data[currentStep] == 1))
			{
				currentBit = 1;
			}
			else
			{
				currentBit = 0;
			}
			
			if(currentBit == 1)
			{
				if(this.getValue("modType") == "nrzl")
				{
					signal = 0;
				}
				else if(this.getValue("modType") == "nrzi")
				{
					if(signal == 1)
					{
						signal = 0;
					}
					else
					{
						signal = 1;
					}
				}
				else if(this.getValue("modType") == "bami")
				{
					if (lastBit == 1)
					{
						signal = -1;
					}
					else
					{
						signal = 1;
					}
					
					lastBit = signal;
				}
				else if(this.getValue("modType") == "pst")
				{
					signal = 0;
				}
				else if(this.getValue("modType") == "manchester")
				{
					signal = 0;
					lastBit = 0;
				}
				else if(this.getValue("modType") == "dManchester")
				{
					lastBit = 0;
				}
			}
			else
			{
				if(this.getValue("modType") == "nrzl")
				{
					signal = 1;
				}
				else if(this.getValue("modType") == "bami")
				{
					signal = 0;
				}
				else if(this.getValue("modType") == "pst")
				{
					if (lastBit == 1)
					{
						signal = -1;
					}
					else
					{
						signal = 1;
					}
					
					lastBit = signal;
				}
				else if(this.getValue("modType") == "manchester")
				{
					signal = 1;
					lastBit = 0;
				}
				else if(this.getValue("modType") == "dManchester")
				{
					if(signal == 1)
					{
						signal = 0;
					}
					else
					{
						signal = 1;
					}
					lastBit = 0;
				}
			}
			signalTarget = signalTarget + signalStep;
			currentStep++;
		}
		
		if ((x >= (signalTarget - (signalStep / 2))) && ((lastBit == 0) && (x != 0)))
		{
			if(currentBit == 1)
			{
				if(this.getValue("modType") == "manchester")
				{
					signal = 1;
				}
				else if(this.getValue("modType") == "dManchester")
				{
					if(signal == 1)
					{
						signal = 0;
					}
					else
					{
						signal = 1;
					}
				}
				lastBit = 1;			
			}
			else
			{
				if(this.getValue("modType") == "manchester")
				{
					signal = 0;
				}
				else if(this.getValue("modType") == "dManchester")
				{
					if(signal == 1)
					{
						signal = 0;
					}
					else
					{
						signal = 1;
					}
				}
				
				lastBit = 1;			
			}
		}
		
		if(((x == 0) || (x == ANALOG_TIME_STEP)) && (this.getValue("modType") == "dManchester"))
		{
			if(signal == 1)
			{
				signal = 0;
			}
			else
			{
				signal = 1;
			}
		}
	
		data.push(
		{
			x: x,
			y: signal
		});
	}
	
	new SignalGraph(this.graphParent, data, "Digital Waveform", false, "Seconds", "Logic Value");	
}



function DigitalSignal(controlParent, graphParent, name, title, storage)
{
	this.name = name;
	this.graphParent = graphParent;
	this.storage = storage;
	
	var dataInput = getTextInput("Data", name, "data", "10010", this.inputsChanged);
	var bps = getTextInput("Data Rate (BPS)", name, "bps", 1, this.inputsChanged);
	
	var panelBox = getTitledBox(title);
	panelBox.appendChild(dataInput);
	panelBox.appendChild(bps);
	
	controlParent.appendChild(panelBox);
	
	var box = getSquareBox("Digital Data Message");
	
	var textContainer = document.createElement("div");
	textContainer.setAttribute("class", "outputText");
	textContainer.setAttribute("id", name + "_" + "outputText");
	box.appendChild(textContainer);
	
	graphParent.appendChild(box);
		
	this.refresh();
}

DigitalSignal.prototype.inputsChanged = function()
{
	updateElements();
};

DigitalSignal.prototype.refresh = function()
{
	document.getElementById(this.name + "_" + "outputText").innerHTML = this.getValue("data");
	
	var storeData = {data: this.getValue("data"), bps: this.getValue("bps")};
	
	this.storeData(storeData);	
};

DigitalSignal.prototype.getValue = function(name)
{
	return document.getElementById(this.name + "_" + name).value;
};

DigitalSignal.prototype.storeData = function(data)
{
	if(this.storage == "data")
	{
		DATASTORAGE = data;
		
	}
	else
	{
		MODSTORAGE = data;
	}
};



function AnalogSignal(controlParent, graphParent, name, title, storage, eqVar)
{
	this.name = name;
	this.title = title;
	this.eqVar = eqVar;
	this.graphParent = graphParent;
	this.storage = storage;
	
	var listData = [{value: "sine", label: "Sine"}, {value: "cosine", label: "Cosine"}];

	var waveInput = getListInput("Wave Type", name, "waveType", listData, this.inputsChanged);
	var freqInput = getTextInput("Frequency (Hz): `" + eqVar + "_f`", name, "freq", 1, this.inputsChanged);
	var ampInput = getTextInput("Amplitude: `" + eqVar + "_a`", name, "amp", 1, this.inputsChanged);
	var eqCont = getContainer(name, "waveData");
	
	
	
	var panelBox = getTitledBox(title);
	panelBox.appendChild(waveInput);
	panelBox.appendChild(freqInput);
	panelBox.appendChild(ampInput);
	panelBox.appendChild(eqCont);
	
	controlParent.appendChild(panelBox);
	
	this.dataCont = document.getElementById(name + "_" + "waveData");
	
	this.refresh();
};

AnalogSignal.prototype.inputsChanged = function()
{
	updateElements();
};

AnalogSignal.prototype.refresh = function()
{
	this.graphParent.innerHTML = "";
	
	var sigEq = "";
	var data = [];
	
	var index = 0;
	for(var x = 0; x <= ANALOG_NUM_CYCLES; x = x + ANALOG_TIME_STEP)
	{
		var calculatedValue = 0;
	
		if(this.getValue("waveType") == "sine")
		{
			calculatedValue = this.getValue("amp") * Math.sin(x * TWOPI * this.getValue("freq"));
			sigEq = this.eqVar + "(t) = " + this.getValue("amp") + " sin(t)";
		}
		else
		{
			calculatedValue = this.getValue("amp") * Math.cos(x * TWOPI * this.getValue("freq"));
			sigEq = this.eqVar + " (t) = " + this.getValue("amp") + " cos(t)";
		}
	
		data[index] = 
		{
			x: x,
			y: calculatedValue
		};
	
		index++;
	}
		
	var storageData = {waveType: this.getValue("waveType"), freq: this.getValue("freq"), amp: this.getValue("amp"), eq: sigEq};
	this.storeData(storageData);
	
	this.dataCont.innerHTML = "";
	var eqRow = getFormRow("Equation");
	eqRow.childNodes[1].appendChild(makeEquation(sigEq));
	this.dataCont.appendChild(eqRow);
	
	new SignalGraph(this.graphParent, data, this.title + " Waveform", true, "Seconds", "Amplitude");	
};

AnalogSignal.prototype.storeData = function(data)
{
	if(this.storage == "data")
	{
		DATASTORAGE = data;
	}
	else
	{
		MODSTORAGE = data;
	}
};

AnalogSignal.prototype.getValue = function(name)
{
	return document.getElementById(this.name + "_" + name).value;
};